
// RecordSignalDlg.cpp : implementation file
//

#include "stdafx.h"
#include "RecordSignal.h"
#include "RecordSignalDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define ID_MY_TIMER  101

CRecordSignalDlg::CRecordSignalDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRecordSignalDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_SP = NULL;
}

void CRecordSignalDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Control(pDX, IDC_START_HZ, m_EDStartHz);
	DDX_Control(pDX, IDC_STOP_HZ, m_EDStopHz);
	DDX_Control(pDX, IDC_THRESHOLD_LVL, m_EDThresholdLvl);
	DDX_Control(pDX, IDC_FREQ_0, m_EDFreq_0);
	DDX_Control(pDX, IDC_LEVEL_0, m_EDLevel_0);
	DDX_Control(pDX, IDC_RECORD_STATUS, m_STRecordStatus);
}

BEGIN_MESSAGE_MAP(CRecordSignalDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_OPEN, &CRecordSignalDlg::OnBnClickedOpen)
	ON_BN_CLICKED(IDC_RUN, &CRecordSignalDlg::OnBnClickedRun)
	ON_BN_CLICKED(IDC_STOP, &CRecordSignalDlg::OnBnClickedStop)
	ON_EN_CHANGE(IDC_START_HZ, &CRecordSignalDlg::OnEnChangeStartHz)
	ON_EN_CHANGE(IDC_STOP_HZ, &CRecordSignalDlg::OnEnChangeStopHz)
	ON_WM_TIMER()
	ON_WM_CLOSE()
END_MESSAGE_MAP()


// CRecordSignalDlg message handlers

BOOL CRecordSignalDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	CoInitialize(NULL);

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_EDStartHz.SetWindowText("100.0");
	m_EDStopHz.SetWindowText("3000.0");
	m_EDThresholdLvl.SetWindowText("-70");

	m_STRecordStatus.SetWindowText("Recording Stopped");

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CRecordSignalDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRecordSignalDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



//------------------------------------------------
void CRecordSignalDlg::OnBnClickedOpen()
{
	CWaitCursor  wait;

	if ( m_SP == NULL)
	{
		m_SP.CreateInstance( __uuidof(SpectraPLUS_SCClass) );
	}

	if ( m_SP == NULL)
	{
		MessageBox("Failure opening SpectraPLUS-SC.", AfxGetAppName(), MB_ICONSTOP);
		return;
	}


	// NOTE - Using a config file with SpectraPLUS-SC is highly recommended
	// m_SP->LoadConfig("C:\\SpectraPLUS_SC\\Config\\test2.cfg");

	//m_SP->SetMode(0);  // Realtime

	//m_SP->SetSamplingRate(48000);
   // m_SP->SetNumChannels(1);
	//m_SP->SetSamplingPrecision(24);

	m_SP->SetMode(1);  // Recorder

}

//------------------------------------------------
void CRecordSignalDlg::OnClose()
{
	KillTimer( ID_MY_TIMER );

	if ( m_SP == NULL)
	{
		m_SP = NULL;  // smart pointer - handles itself
	}

	CDialog::OnClose();
}

//------------------------------------------------
void CRecordSignalDlg::OnBnClickedRun()
{
	if ( m_SP == NULL)
	{
		MessageBox("SpectraPLUS-SC is not open", AfxGetAppName(), MB_ICONSTOP);
		return;
	}

	m_SP->Record();
	m_SP->RecordPause();

	KillTimer( ID_MY_TIMER );
	SetTimer( ID_MY_TIMER, 100, NULL );  // 500 msec update rate
}

//------------------------------------------------
void CRecordSignalDlg::OnBnClickedStop()
{
	KillTimer( ID_MY_TIMER );

	if ( m_SP == NULL)
	{
		MessageBox("SpectraPLUS-SC is not open", AfxGetAppName(), MB_ICONSTOP);
		return;
	}

	m_SP->Stop();
	m_STRecordStatus.SetWindowText("Recording Stopped");
}

//------------------------------------------------
void CRecordSignalDlg::OnEnChangeStartHz()
{
	GetResults();
}

//------------------------------------------------
void CRecordSignalDlg::OnEnChangeStopHz()
{
	GetResults();
}

//------------------------------------------------
void CRecordSignalDlg::OnTimer(UINT_PTR nIDEvent)
{
	CDialog::OnTimer(nIDEvent);

	GetResults();
}


//------------------------------------------------
void CRecordSignalDlg::GetResults()
{
	if ( m_SP == NULL) return;

	CString szStart;
	m_EDStartHz.GetWindowText(szStart);
	float startHz = (float)atof(szStart);

	CString szStop;
	m_EDStopHz.GetWindowText(szStop);
	float stopHz = (float)atof(szStop);

	CString szThreshold;
	m_EDThresholdLvl.GetWindowText(szThreshold);
	float thresholdLvl = (float)atof(szThreshold);

	int peak_number = 0;

	try
	{
		CString szFreq;

		float pk_freq = 0.0;
		m_SP->GetPeakFrequencyNthSpan(0, peak_number, startHz, stopHz, &pk_freq);
		szFreq.Format("%f", pk_freq);
		m_EDFreq_0.SetWindowText(szFreq);

		CString szLevel;

		float pk_level = 0.0;
		m_SP->GetPeakAmplitudeNthSpan(0, peak_number, startHz, stopHz, &pk_level);
		szLevel.Format("%f", pk_level);
		m_EDLevel_0.SetWindowText(szLevel);

		if (pk_level > thresholdLvl)
		{
			m_SP->RecordResume();
			m_STRecordStatus.SetWindowText("Signal Detected - Recording...");
		}
		else
		{
			m_SP->RecordPause();
			m_STRecordStatus.SetWindowText("No Signal - Recording Paused...");
		}
	}
	catch ( _com_error &error )
	{
		KillTimer( ID_MY_TIMER );
		MessageBox( error.ErrorMessage() );
	}
	
}
